// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.strategy.merge.IMergable;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorQuery;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.CdmBaseSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.TitleCacheComparator;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>Abstract AbstractBulkEditorInput class.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractBulkEditorInput<T extends ICdmBase> implements IEditorInput, IEntityPersistenceService<T> {

	private UUID entityUuid;

	private List<T> model;

	private IEntityCreator<T> entityCreator;

	private static Class serviceClass;

	/**
	 * <p>NewInstance</p>
	 *
	 * @param inputType a {@link eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputTypeValues.BulkEditorInputType} object.
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	static public AbstractBulkEditorInput NewInstance(BulkEditorInputType inputType) {

		return BulkEditorInputType.getInput(inputType);
	}

	/**
	 * <p>NewInstance</p>
	 *
	 * @param entity a {@link eu.etaxonomy.cdm.model.common.IdentifiableEntity} object.
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput NewInstance(IdentifiableEntity entity) {


		BulkEditorInputType inputType = BulkEditorInputType.getByType(entity.getClass());

		AbstractBulkEditorInput editorInput = NewInstance(inputType);

		editorInput.setEntityUuid(entity.getUuid());

		return editorInput;
	}

	/**
	 * <p>listEntities</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	protected abstract List<T> listEntities(IIdentifiableEntityServiceConfigurator configurator);

	/**
	 * <p>loadEntity</p>
	 *
	 * @param entityUuid a {@link java.util.UUID} object.
	 * @return a T object.
	 */
	protected T loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return (T) CdmStore.getService(serviceClass).load(entityUuid, propertyPaths);
	}

	private void setEntityUuid(UUID entityUuid){
		this.entityUuid = entityUuid;
	}

	/**
	 * <p>Getter for the field <code>entityUuid</code>.</p>
	 *
	 * @return a {@link java.util.UUID} object.
	 */
	public UUID getEntityUuid() {
		return entityUuid;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	/**
	 * <p>exists</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean exists() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	/**
	 * <p>getImageDescriptor</p>
	 *
	 * @return a {@link org.eclipse.jface.resource.ImageDescriptor} object.
	 */
	@Override
    public ImageDescriptor getImageDescriptor() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	/**
	 * <p>getPersistable</p>
	 *
	 * @return a {@link org.eclipse.ui.IPersistableElement} object.
	 */
	@Override
    public IPersistableElement getPersistable() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	/** {@inheritDoc} */
	@Override
    @SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * <p>Setter for the field <code>query</code>.</p>
	 *
	 * @param bulkEditorQuery a {@link eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery} object.
	 */
	public void performSearch(final BulkEditorQuery bulkEditorQuery) {

		List<T> entityList = new ArrayList<T>();

		if(getEntityUuid() != null){

			T entity = loadEntity(getEntityUuid());
			entityList.add(entity);
			model = entityList;
		}
		else if(bulkEditorQuery != null){

			IIdentifiableEntityServiceConfigurator configurator = bulkEditorQuery.getSearchConfigurator();
			Comparator queryComparator = (bulkEditorQuery.getComparator() != null) ? bulkEditorQuery.getComparator() : new TitleCacheComparator();

			entityList = listEntities(configurator);

			Collections.sort(entityList, queryComparator);


		}

		model = entityList;
	}

	/**
	 * <p>isMergingEnabled</p>
	 *
	 * @return a boolean.
	 */
	public boolean isMergingEnabled() {
		return false;
	}

	/**
	 * <p>isMergingEnabled</p>
	 *
	 * @return a boolean.
	 */
	public boolean isConvertingEnabled() {
		return false;
	}
	/**
	 * <p>isMarkerTypeEditingEnabled</p>
	 *
	 * @param markerType a {@link eu.etaxonomy.cdm.model.common.MarkerType} object.
	 * @return a boolean.
	 */
	public boolean isMarkerTypeEditingEnabled(MarkerType markerType) {
		return false;
	}


	/** {@inheritDoc} */
	@Override
    public boolean merge(T entity, T mergeTarget) {
		if (entity instanceof IMergable) {
			try {
				CdmStore.getCommonService().merge(mergeTarget.getId(), entity.getId(), (Class<? extends CdmBase>)entity.getClass());
			} catch (MergeException e) {
				MessagingUtils.errorDialog("Bulk Editor Merge Error",
						this,
						"Could not merge chosen objects of type " + entity.getClass().getName(),
						TaxeditorBulkeditorPlugin.PLUGIN_ID,
						e,
						true);
			}
		}
		return true;
	}


	/** {@inheritDoc} */
	@Override
    public boolean create(T entity) {
		return save(entity);
	}

	public IEntityCreator<T> getEntityCreator(){
		if(entityCreator == null){
			entityCreator = createEntityCreator();
		}
		return entityCreator;
	}

	/**
	 * @return
	 */
	protected abstract IEntityCreator<T> createEntityCreator();

	/**
	 * The default implementation returns an empty list of sort providers.
	 * @return
	 */
	public List<IBulkEditorSortProvider<T>> getSortProviders(){
		List<IBulkEditorSortProvider<T>> sortProviders = new ArrayList<IBulkEditorSortProvider<T>>();

		sortProviders.add(new CdmBaseSortProvider<T>());

		return sortProviders;
	}

	/**
	 * Returns a textual representation given object. The default implementation
	 * in the abstract base class returns the simple name of the class, this may
	 * be overwritten to something more specific in subclasses.
	 *
	 * @param entity
	 * @return a textual representation given object.
	 */
	public String getTypeText(Object entity){
		return entity.getClass().getSimpleName();
	}

	/**
	 * @param entity
	 * @return
	 */
	public String getText(T entity) {
		if(entity instanceof IdentifiableEntity){
			IdentifiableEntity identifiableEntity = (IdentifiableEntity) HibernateProxyHelper.deproxy(entity);

			return identifiableEntity.getTitleCache();
		}

		return "No text. Implement in subclass";
	}

	/**
	 * @return
	 */
	public List<?> getModel() {
		return model;
	}


}
