// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.UserCreator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 9, 2011
 * @version 1.0
 */
public class UserEditorInput extends AbstractBulkEditorInput<User> {

	public static final String ID = "bulkeditor.input.user";
	
	private static UserEditorInput instance;
	
	/**
	 * @return the instance
	 */
	public static UserEditorInput getInstance() {
		if(instance == null){
			instance = new UserEditorInput();
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	@Override
	public String getName() {
		return BulkEditorInputType.USER.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	@Override
	public String getToolTipText() {
		return getName();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#save(java.lang.Object)
	 */
	@Override
	public boolean save(User entity) {
		return CdmStore.getService(IUserService.class).saveOrUpdate(entity) != null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#delete(java.lang.Object)
	 */
	@Override
	public boolean delete(User entity) throws ReferencedObjectUndeletableException {
		return CdmStore.getService(IUserService.class).delete(entity) != null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#listEntities(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
	 */
	@Override
	protected List<User> listEntities(
			IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.getSearchManager().findUsers(configurator);
	}

	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#loadEntity(java.util.UUID)
	
	@Override
	protected User loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{}); 
		return CdmStore.getService(IUserService.class).load(entityUuid, propertyPaths);
	}
 */
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#createEntityCreator()
	 */
	@Override
	protected IEntityCreator<User> createEntityCreator() {
		return new UserCreator();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getText(eu.etaxonomy.cdm.model.common.ICdmBase)
	 */
	@Override
	public String getText(User entity) {
		return String.format("%s, %s", entity.getUsername(), entity.getPerson());
	}


}
