// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.common.IAnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * @author n.hoffmann
 * @created Dec 13, 2010
 * @version 1.0
 */
public class SetMarkerFlagOperation extends AbstractPostTaxonOperation {

	private Set<IAnnotatableEntity> annotatableEntities;
	private MarkerType markerType;
	private boolean value;

	/**
	 * @param label
	 * @param undoContext
	 * @param postOperationEnabled
	 */
	public SetMarkerFlagOperation(String label, IUndoContext undoContext,
			Set<IAnnotatableEntity> annotatableEntities, MarkerType markerType, boolean value,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.annotatableEntities = annotatableEntities;
		this.markerType = markerType;
		this.value = value;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		for(IAnnotatableEntity annotatableEntity : annotatableEntities){
			Marker marker = Marker.NewInstance(markerType, value);
			
			annotatableEntity.addMarker(marker);
		}
		
		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		for(IAnnotatableEntity annotatableEntity : annotatableEntities){
			Marker marker = Marker.NewInstance(markerType, ! value);
			
			annotatableEntity.addMarker(marker);
		}
		
		return postExecute(null);
	}

}
